@@ -63,19 +63,20 @@ class GroupDefImpl : public DefinitionMixin<GroupDef>
     virtual QCString groupTitle() const { return m_title; }
     virtual void setGroupTitle( const QCString &newtitle );
     virtual bool hasGroupTitle( ) const { return m_titleSet; }
-    virtual void addFile(const FileDef *def);
-    virtual bool addClass(const ClassDef *def);
-    virtual bool addConcept(const ConceptDef *def);
-    virtual bool addNamespace(const NamespaceDef *def);
-    virtual void addGroup(const GroupDef *def);
-    virtual void addPage(const PageDef *def);
-    virtual void addExample(const PageDef *def);
+    virtual void addFile(FileDef *def);
+    virtual bool containsFile(const FileDef *def) const;
+    virtual bool addClass(ClassDef *def);
+    virtual bool addConcept(ConceptDef *def);
+    virtual bool addNamespace(NamespaceDef *def);
+    virtual void addGroup(GroupDef *def);
+    virtual void addPage(PageDef *def);
+    virtual void addExample(PageDef *def);
     virtual void addDir(DirDef *dd);
-    virtual bool insertMember(const MemberDef *def,bool docOnly=FALSE);
+    virtual bool insertMember(MemberDef *def,bool docOnly=FALSE);
     virtual void removeMember(MemberDef *md);
     virtual bool findGroup(const GroupDef *def) const; // true if def is a subgroup of this group
     virtual void writeDocumentation(OutputList &ol);
-    virtual void writeMemberPages(OutputList &ol);
+    virtual void writeMemberPages(OutputList &ol, int hierarchyLevel);
     virtual void writeQuickMemberLinks(OutputList &ol,const MemberDef *currentMd) const;
     virtual void writeTagFile(TextStream &);
     virtual size_t numDocMembers() const;
@@ -115,7 +116,7 @@ class GroupDefImpl : public DefinitionMixin<GroupDef>
 
   private:
     void addMemberListToGroup(MemberList *,bool (MemberDef::*)() const);
-    void addMemberToList(MemberListType lt,const MemberDef *md);
+    void addMemberToList(MemberListType lt,MemberDef *md);
     void writeMemberDeclarations(OutputList &ol,MemberListType lt,const QCString &title);
     void writeMemberDocumentation(OutputList &ol,MemberListType lt,const QCString &title);
     void removeMemberFromList(MemberListType lt,MemberDef *md);
@@ -159,10 +160,10 @@ class GroupDefImpl : public DefinitionMixin<GroupDef>
 
 };
 
-GroupDef *createGroupDef(const QCString &fileName,int line,const QCString &name,
+std::unique_ptr<GroupDef> createGroupDef(const QCString &fileName,int line,const QCString &name,
                                 const QCString &title,const QCString &refFileName)
 {
-  return new GroupDefImpl(fileName,line,name,title,refFileName);
+  return std::make_unique<GroupDefImpl>(fileName,line,name,title,refFileName);
 }
 
 
@@ -234,7 +235,7 @@ void GroupDefImpl::findSectionsInDocumentation()
   }
 }
 
-void GroupDefImpl::addFile(const FileDef *def)
+void GroupDefImpl::addFile(FileDef *def)
 {
   bool sortBriefDocs = Config_getBool(SORT_BRIEF_DOCS);
   if (def->isHidden()) return;
@@ -248,7 +249,12 @@ void GroupDefImpl::addFile(const FileDef *def)
     m_fileList.push_back(def);
 }
 
-bool GroupDefImpl::addClass(const ClassDef *cd)
+bool GroupDefImpl::containsFile(const FileDef *def) const
+{
+  return std::find(m_fileList.cbegin(),m_fileList.cend(), def) != m_fileList.cend();
+}
+
+bool GroupDefImpl::addClass(ClassDef *cd)
 {
   if (cd->isHidden()) return FALSE;
   updateLanguage(cd);
@@ -261,7 +267,7 @@ bool GroupDefImpl::addClass(const ClassDef *cd)
   return FALSE;
 }
 
-bool GroupDefImpl::addConcept(const ConceptDef *cd)
+bool GroupDefImpl::addConcept(ConceptDef *cd)
 {
   if (cd->isHidden()) return FALSE;
   QCString qn = cd->name();
@@ -273,7 +279,7 @@ bool GroupDefImpl::addConcept(const ConceptDef *cd)
   return FALSE;
 }
 
-bool GroupDefImpl::addNamespace(const NamespaceDef *def)
+bool GroupDefImpl::addNamespace(NamespaceDef *def)
 {
   //printf("adding namespace hidden=%d\n",def->isHidden());
   if (def->isHidden()) return false;
@@ -292,15 +298,15 @@ void GroupDefImpl::addDir(DirDef *def)
   m_dirList.push_back(def);
 }
 
-void GroupDefImpl::addPage(const PageDef *def)
+void GroupDefImpl::addPage(PageDef *def)
 {
   if (def->isHidden()) return;
   //printf("Making page %s part of a group\n",qPrint(def->name));
   m_pages.add(def->name(),def);
-  const_cast<PageDef*>(def)->makePartOfGroup(this);
+  def->makePartOfGroup(this);
 }
 
-void GroupDefImpl::addExample(const PageDef *def)
+void GroupDefImpl::addExample(PageDef *def)
 {
   if (def->isHidden()) return;
   m_examples.add(def->name(),def);
@@ -319,7 +325,7 @@ void GroupDefImpl::addMembersToMemberGroup()
 }
 
 
-bool GroupDefImpl::insertMember(const MemberDef *md,bool docOnly)
+bool GroupDefImpl::insertMember(MemberDef *md,bool docOnly)
 {
   if (md->isHidden()) return FALSE;
   updateLanguage(md);
@@ -416,7 +422,7 @@ bool GroupDefImpl::insertMember(const MemberDef *md,bool docOnly)
       addMemberToList(MemberListType_docSignalMembers,md);
       break;
     case MemberType_Slot:
-      if (md->protection()==Public)
+      if (md->protection()==Protection::Public)
       {
         if (!docOnly)
         {
@@ -424,7 +430,7 @@ bool GroupDefImpl::insertMember(const MemberDef *md,bool docOnly)
         }
         addMemberToList(MemberListType_docPubSlotMembers,md);
       }
-      else if (md->protection()==Protected)
+      else if (md->protection()==Protection::Protected)
       {
         if (!docOnly)
         {
@@ -512,12 +518,12 @@ void GroupDefImpl::removeMember(MemberDef *md)
         removeMemberFromList(MemberListType_docSignalMembers,md);
         break;
       case MemberType_Slot:
-        if (md->protection()==Public)
+        if (md->protection()==Protection::Public)
         {
           removeMemberFromList(MemberListType_decPubSlotMembers,md);
           removeMemberFromList(MemberListType_docPubSlotMembers,md);
         }
-        else if (md->protection()==Protected)
+        else if (md->protection()==Protection::Protected)
         {
           removeMemberFromList(MemberListType_decProSlotMembers,md);
           removeMemberFromList(MemberListType_docProSlotMembers,md);
@@ -562,7 +568,7 @@ bool GroupDefImpl::findGroup(const GroupDef *def) const
   return FALSE;
 }
 
-void GroupDefImpl::addGroup(const GroupDef *def)
+void GroupDefImpl::addGroup(GroupDef *def)
 {
   //printf("adding group '%s' to group '%s'\n",qPrint(def->name()),qPrint(name()));
   //if (Config_getBool(SORT_MEMBER_DOCS))
@@ -610,10 +616,12 @@ void GroupDefImpl::computeAnchors()
 
 void GroupDefImpl::writeTagFile(TextStream &tagFile)
 {
+  QCString fn = getOutputFileBase();
+  addHtmlExtensionIfMissing(fn);
   tagFile << "  <compound kind=\"group\">\n";
   tagFile << "    <name>" << convertToXML(name()) << "</name>\n";
   tagFile << "    <title>" << convertToXML(m_title) << "</title>\n";
-  tagFile << "    <filename>" << addHtmlExtensionIfMissing(getOutputFileBase()) << "</filename>\n";
+  tagFile << "    <filename>" << fn << "</filename>\n";
   for (const auto &lde : LayoutDocManager::instance().docEntries(LayoutDocManager::Group))
   {
     switch (lde->kind())
@@ -768,7 +776,6 @@ void GroupDefImpl::writeDetailedDescription(OutputList &ol,const QCString &title
       ol.pushGeneratorState();
       ol.disable(OutputType::Man);
       ol.disable(OutputType::RTF);
-      // ol.newParagraph(); // FIXME:PARA
       ol.enableAll();
       ol.disableAllBut(OutputType::Man);
       ol.enable(OutputType::Latex);
@@ -861,11 +868,11 @@ void GroupDefImpl::writeFiles(OutputList &ol,const QCString &title)
     {
       if (!fd->hasDocumentation()) continue;
       ol.startMemberDeclaration();
-      ol.startMemberItem(fd->getOutputFileBase(),0);
+      ol.startMemberItem(fd->getOutputFileBase(),OutputGenerator::MemberItemType::Normal);
       ol.docify(theTranslator->trFile(FALSE,TRUE)+" ");
       ol.insertMemberAlign();
       ol.writeObjectLink(fd->getReference(),fd->getOutputFileBase(),QCString(),fd->name());
-      ol.endMemberItem();
+      ol.endMemberItem(OutputGenerator::MemberItemType::Normal);
       if (!fd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
       {
         ol.startMemberDescription(fd->getOutputFileBase());
@@ -905,12 +912,12 @@ void GroupDefImpl::writeNestedGroups(OutputList &ol,const QCString &title)
       {
         if (!gd->hasDocumentation()) continue;
         ol.startMemberDeclaration();
-        ol.startMemberItem(gd->getOutputFileBase(),0);
+        ol.startMemberItem(gd->getOutputFileBase(),OutputGenerator::MemberItemType::Normal);
         //ol.docify(theTranslator->trGroup(FALSE,TRUE));
         //ol.docify(" ");
         ol.insertMemberAlign();
         ol.writeObjectLink(gd->getReference(),gd->getOutputFileBase(),QCString(),gd->groupTitle());
-        ol.endMemberItem();
+        ol.endMemberItem(OutputGenerator::MemberItemType::Normal);
         if (!gd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
         {
           ol.startMemberDescription(gd->getOutputFileBase());
@@ -938,11 +945,11 @@ void GroupDefImpl::writeDirs(OutputList &ol,const QCString &title)
     {
       if (!dd->hasDocumentation()) continue;
       ol.startMemberDeclaration();
-      ol.startMemberItem(dd->getOutputFileBase(),0);
+      ol.startMemberItem(dd->getOutputFileBase(),OutputGenerator::MemberItemType::Normal);
       ol.parseText(theTranslator->trDir(FALSE,TRUE));
       ol.insertMemberAlign();
       ol.writeObjectLink(dd->getReference(),dd->getOutputFileBase(),QCString(),dd->shortName());
-      ol.endMemberItem();
+      ol.endMemberItem(OutputGenerator::MemberItemType::Normal);
       if (!dd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
       {
         ol.startMemberDescription(dd->getOutputFileBase());
@@ -1102,7 +1109,18 @@ void GroupDefImpl::writeDocumentation(OutputList &ol)
 {
   //bool generateTreeView = Config_getBool(GENERATE_TREEVIEW);
   ol.pushGeneratorState();
-  startFile(ol,getOutputFileBase(),name(),m_title,HighlightedItem::Modules);
+
+  // Find out how deep this group is nested. In case of multiple parents, use the first one.
+  int hierarchyLevel = 0;
+  const GroupDef *gd = this;
+  while (!gd->partOfGroups().empty())
+  {
+    gd = gd->partOfGroups().front();
+    ++hierarchyLevel;
+  }
+
+  startFile(ol,getOutputFileBase(),name(),m_title,HighlightedItem::Modules,
+            FALSE /* additionalIndices*/, QCString() /*altSidebarName*/, hierarchyLevel);
 
   ol.startHeaderSection();
   writeSummaryLinks(ol);
@@ -1244,6 +1262,21 @@ void GroupDefImpl::writeDocumentation(OutputList &ol)
 
   //---------------------------------------- end flexible part -------------------------------
 
+  for (auto &subgd : getSubGroups())
+  {
+    if (!subgd->isReference())
+    {
+      if (subgd->partOfGroups().front() == this)
+      {
+        ol.writePageLink(subgd->getOutputFileBase(), FALSE);
+      }
+      else
+      {
+        // Could write a note explaining that the subgroup belongs to another
+        // group and add a link here.
+      }
+    }
+  }
   endFile(ol);
 
   ol.popGeneratorState();
@@ -1251,12 +1284,11 @@ void GroupDefImpl::writeDocumentation(OutputList &ol)
   if (Config_getBool(SEPARATE_MEMBER_PAGES))
   {
     m_allMemberList.sort();
-    writeMemberPages(ol);
+    writeMemberPages(ol, hierarchyLevel + 1);
   }
-
 }
 
-void GroupDefImpl::writeMemberPages(OutputList &ol)
+void GroupDefImpl::writeMemberPages(OutputList &ol, int hierarchyLevel)
 {
   ol.pushGeneratorState();
   ol.disableAllBut(OutputType::Html);
@@ -1265,7 +1297,7 @@ void GroupDefImpl::writeMemberPages(OutputList &ol)
   {
     if (ml->listType()&MemberListType_documentationLists)
     {
-       ml->writeDocumentationPage(ol,name(),this);
+       ml->writeDocumentationPage(ol,name(),this,hierarchyLevel);
     }
   }
 
@@ -1285,6 +1317,8 @@ void GroupDefImpl::writeQuickMemberLinks(OutputList &ol,const MemberDef *current
     {
       if (md->isLinkableInProject())
       {
+        QCString fn = md->getOutputFileBase();
+        addHtmlExtensionIfMissing(fn);
         if (md==currentMd) // selected item => highlight
         {
           ol.writeString("          <tr><td class=\"navtabHL\">");
@@ -1296,7 +1330,7 @@ void GroupDefImpl::writeQuickMemberLinks(OutputList &ol,const MemberDef *current
         ol.writeString("<a class=\"navtab\" ");
         ol.writeString("href=\"");
         if (createSubDirs) ol.writeString("../../");
-        ol.writeString(addHtmlExtensionIfMissing(md->getOutputFileBase())+"#"+md->anchor());
+        ol.writeString(fn+"#"+md->anchor());
         ol.writeString("\">");
         ol.writeString(convertToHtml(md->localName()));
         ol.writeString("</a>");
@@ -1574,7 +1608,7 @@ void GroupDefImpl::addListReferences()
   }
 }
 
-void GroupDefImpl::addMemberToList(MemberListType lt,const MemberDef *md)
+void GroupDefImpl::addMemberToList(MemberListType lt,MemberDef *md)
 {
   bool sortBriefDocs = Config_getBool(SORT_BRIEF_DOCS);
   bool sortMemberDocs = Config_getBool(SORT_MEMBER_DOCS);
