@@ -200,7 +200,7 @@ void startTitle(OutputList &ol,const QCString &fileName,const DefinitionMutable
 void endTitle(OutputList &ol,const QCString &fileName,const QCString &name);
 void startFile(OutputList &ol,const QCString &name,const QCString &manName,
                const QCString &title,HighlightedItem hli=HighlightedItem::None,
-               bool additionalIndices=FALSE,const QCString &altSidebarName=QCString());
+               bool additionalIndices=FALSE,const QCString &altSidebarName=QCString(), int hierarchyLevel=0);
 void endFile(OutputList &ol,bool skipNavIndex=FALSE,bool skipEndContents=FALSE,
              const QCString &navPath=QCString());
 void endFileWithNavPath(OutputList &ol,const Definition *d);
