@@ -13,51 +13,85 @@
  *
  */
 
-#include <stdio.h>
+#include <cstdio>
+#include <cstdlib>
+#include <mutex>
+#include <atomic>
+
 #include "config.h"
 #include "debug.h"
 #include "portable.h"
 #include "message.h"
 #include "doxygen.h"
-
-#include <mutex>
+#include "fileinfo.h"
+#include "dir.h"
 
 // globals
 static QCString        g_warnFormat;
+static QCString        g_warnLineFormat;
 static const char *    g_warningStr = "warning: ";
 static const char *    g_errorStr = "error: ";
 static FILE *          g_warnFile = stderr;
 static WARN_AS_ERROR_t g_warnBehavior = WARN_AS_ERROR_t::NO;
-static bool            g_warnStat = false;
+static QCString        g_warnlogFile;
+static bool            g_warnlogTemp = false;
+static std::atomic_bool g_warnStat = false;
 static std::mutex      g_mutex;
 
 void initWarningFormat()
 {
   g_warnFormat = Config_getString(WARN_FORMAT);
-  QCString logFile = Config_getString(WARN_LOGFILE);
-
-  if (!logFile.isEmpty())
+  g_warnLineFormat = Config_getString(WARN_LINE_FORMAT);
+  g_warnBehavior = Config_getEnum(WARN_AS_ERROR);
+  g_warnlogFile = Config_getString(WARN_LOGFILE);
+  if (g_warnlogFile.isEmpty() && g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS_PRINT)
   {
-    if (logFile == "-")
+    uint32_t pid = Portable::pid();
+    g_warnlogFile.sprintf("doxygen_warnings_temp_%d.tmp",pid);
+    g_warnlogTemp = true;
+  }
+
+  if (!g_warnlogFile.isEmpty())
+  {
+    if (g_warnlogFile == "-")
     {
       g_warnFile = stdout;
     }
-    else if (!(g_warnFile = Portable::fopen(logFile,"w")))
+    else
     {
-      // point it to something valid, because warn() relies on it
-      g_warnFile = stderr;
-      err("Cannot open '%s' for writing, redirecting 'WARN_LOGFILE' output to 'stderr'\n",logFile.data());
+      FileInfo fi(g_warnlogFile.str());
+      Dir d(fi.dirPath().c_str());
+      if (!d.exists() && !d.mkdir(fi.dirPath().c_str()))
+      {
+        // point it to something valid, because warn() relies on it
+        g_warnFile = stderr;
+        err("Cannot create directory for '%s', redirecting 'WARN_LOGFILE' output to 'stderr'\n",g_warnlogFile.data());
+      }
+      else if (!(g_warnFile = Portable::fopen(g_warnlogFile,"w")))
+      {
+        // point it to something valid, because warn() relies on it
+        g_warnFile = stderr;
+        err("Cannot open '%s' for writing, redirecting 'WARN_LOGFILE' output to 'stderr'\n",g_warnlogFile.data());
+      }
     }
   }
   else
   {
     g_warnFile = stderr;
   }
-  g_warnBehavior = Config_getEnum(WARN_AS_ERROR);
   if (g_warnBehavior != WARN_AS_ERROR_t::NO)
   {
     g_warningStr = g_errorStr;
   }
+
+  // make sure the g_warnFile is closed in case we call exit and it is still open
+  std::atexit([](){
+      if (g_warnFile && g_warnFile!=stderr && g_warnFile!=stdout)
+      {
+        Portable::fclose(g_warnFile);
+        g_warnFile = nullptr;
+      }
+  });
 }
 
 
@@ -68,7 +102,7 @@ void msg(const char *fmt, ...)
     std::unique_lock<std::mutex> lock(g_mutex);
     if (Debug::isFlagSet(Debug::Time))
     {
-      printf("%.3f sec: ",((double)Debug::elapsedTime()));
+      printf("%.3f sec: ",(static_cast<double>(Debug::elapsedTime())));
     }
     va_list args;
     va_start(args, fmt);
@@ -111,6 +145,7 @@ static void format_warn(const QCString &file,int line,const QCString &text)
   }
   if (g_warnBehavior == WARN_AS_ERROR_t::YES)
   {
+    Doxygen::terminating=true;
     exit(1);
   }
   g_warnStat = true;
@@ -125,6 +160,7 @@ static void handle_warn_as_error()
       QCString msgText = " (warning treated as error, aborting now)\n";
       fwrite(msgText.data(),1,msgText.length(),g_warnFile);
     }
+    Doxygen::terminating=true;
     exit(1);
   }
   g_warnStat = true;
@@ -137,28 +173,40 @@ static void do_warn(bool enabled, const QCString &file, int line, const char *pr
   va_list argsCopy;
   va_copy(argsCopy, args);
 
-  int l=0;
+  size_t l=0;
   if (prefix)
   {
-    l=(int)strlen(prefix);
+    l=strlen(prefix);
   }
   // determine needed buffersize based on:
   // format + arguments
   // prefix
   // 1 position for `\0`
-  int bufSize = vsnprintf(NULL, 0, fmt, args) + l + 1;
+  size_t bufSize = vsnprintf(NULL, 0, fmt, args) + l + 1;
   QCString text(bufSize);
   if (prefix)
   {
     qstrncpy(text.rawData(),prefix,bufSize);
   }
   vsnprintf(text.rawData()+l, bufSize-l, fmt, argsCopy);
-  text[bufSize-1]='\0';
+  text[static_cast<int>(bufSize)-1]='\0';
   format_warn(file,line,text);
 
   va_end(argsCopy);
 }
 
+QCString warn_line(const QCString &file,int line)
+{
+  QCString fileSubst = file.isEmpty() ? "<unknown>" : file;
+  QCString lineSubst; lineSubst.setNum(line);
+  return  substitute(
+            substitute(
+              g_warnLineFormat,
+              "$file",fileSubst
+            ),
+            "$line",lineSubst
+          );
+}
 void warn(const QCString &file,int line,const char *fmt, ...)
 {
   va_list args;
@@ -238,10 +286,12 @@ void term(const char *fmt, ...)
     va_end(args);
     if (g_warnFile != stderr)
     {
-      for (int i = 0; i < (int)strlen(g_errorStr); i++) fprintf(g_warnFile, " ");
+      size_t l = strlen(g_errorStr);
+      for (size_t i=0; i<l; i++) fprintf(g_warnFile, " ");
       fprintf(g_warnFile, "%s\n", "Exiting...");
     }
   }
+  Doxygen::terminating=true;
   exit(1);
 }
 
@@ -251,38 +301,40 @@ void warn_flush()
 }
 
 
-void printlex(int dbg, bool enter, const char *lexName, const char *fileName)
-{
-  const char *enter_txt = "entering";
-  const char *enter_txt_uc = "Entering";
-
-  if (!enter)
-  {
-    enter_txt = "finished";
-    enter_txt_uc = "Finished";
-  }
-
-  std::unique_lock<std::mutex> lock(g_mutex);
-  if (dbg)
-  {
-    if (fileName)
-      fprintf(stderr,"--%s lexical analyzer: %s (for: %s)\n",enter_txt, qPrint(lexName), qPrint(fileName));
-    else
-      fprintf(stderr,"--%s lexical analyzer: %s\n",enter_txt, qPrint(lexName));
-  }
-  else
-  {
-    if (fileName)
-      Debug::print(Debug::Lex,0,"%s lexical analyzer: %s (for: %s)\n",enter_txt_uc, qPrint(lexName), qPrint(fileName));
-    else
-      Debug::print(Debug::Lex,0,"%s lexical analyzer: %s\n",enter_txt_uc, qPrint(lexName));
-  }
-}
 
 extern void finishWarnExit()
 {
-  if (g_warnStat && g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS)
+  if (g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS_PRINT && g_warnlogFile != "-")
   {
+    Portable::fclose(g_warnFile);
+    g_warnFile = nullptr;
+  }
+  if (g_warnStat && g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS_PRINT && g_warnlogFile != "-")
+  {
+
+    std::ifstream warnFile = Portable::openInputStream(g_warnlogFile);
+    if (!warnFile.is_open())
+    {
+      g_warnFile = stderr;
+      err("Cannot open warnings file '%s' for reading\n",g_warnlogFile.data());
+    }
+    else
+    {
+      std::string line;
+      while (getline(warnFile,line))
+      {
+        fprintf(stderr,"%s\n",line.c_str());
+      }
+      warnFile.close();
+    }
+  }
+
+  if (g_warnlogTemp) Portable::unlink(g_warnlogFile);
+
+  if (g_warnStat && (g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS ||
+                     g_warnBehavior == WARN_AS_ERROR_t::FAIL_ON_WARNINGS_PRINT))
+  {
+    Doxygen::terminating=true;
     exit(1);
   }
 }
