@@ -67,20 +67,19 @@ class NamespaceDefImpl : public DefinitionMixin<NamespaceDefMutable>
     virtual void writeMemberPages(OutputList &ol);
     virtual void writeQuickMemberLinks(OutputList &ol,const MemberDef *currentMd) const;
     virtual void writeTagFile(TextStream &);
-    virtual void insertClass(const ClassDef *cd);
-    virtual void insertConcept(const ConceptDef *cd);
-    virtual void insertNamespace(const NamespaceDef *nd);
+    virtual void insertClass(ClassDef *cd);
+    virtual void insertConcept(ConceptDef *cd);
+    virtual void insertNamespace(NamespaceDef *nd);
     virtual void insertMember(MemberDef *md);
     virtual void computeAnchors();
     virtual void countMembers();
     virtual int  numDocMembers() const;
-    virtual void addUsingDirective(const NamespaceDef *nd);
-    virtual const LinkedRefMap<const NamespaceDef> &getUsedNamespaces() const { return m_usingDirList; }
-    virtual void addUsingDeclaration(const ClassDef *cd);
-    virtual const LinkedRefMap<const ClassDef> &getUsedClasses() const { return m_usingDeclList; }
+    virtual void addUsingDirective(NamespaceDef *nd);
+    virtual const LinkedRefMap<NamespaceDef> &getUsedNamespaces() const { return m_usingDirList; }
+    virtual void addUsingDeclaration(ClassDef *cd);
+    virtual const LinkedRefMap<ClassDef> &getUsedClasses() const { return m_usingDeclList; }
     virtual void combineUsingRelations(NamespaceDefSet &visitedNamespace);
     virtual QCString displayName(bool=TRUE) const;
-    virtual QCString localName() const;
     virtual void setInline(bool isInline) { m_inline = isInline; }
     virtual bool isConstantGroup() const { return CONSTANT_GROUP == m_type; }
     virtual bool isModule()        const { return NAMESPACE == m_type || MODULE == m_type; }
@@ -94,7 +93,7 @@ class NamespaceDefImpl : public DefinitionMixin<NamespaceDefMutable>
     virtual void findSectionsInDocumentation();
     virtual void sortMemberLists();
     virtual const Definition *findInnerCompound(const QCString &name) const;
-    virtual void addInnerCompound(const Definition *d);
+    virtual void addInnerCompound(Definition *d);
     virtual void addListReferences();
     virtual void setFileName(const QCString &fn);
     virtual bool subGrouping() const { return m_subGrouping; }
@@ -108,6 +107,7 @@ class NamespaceDefImpl : public DefinitionMixin<NamespaceDefMutable>
     virtual ClassLinkedRefMap getExceptions() const { return exceptions; }
     virtual NamespaceLinkedRefMap getNamespaces() const { return namespaces; }
     virtual ConceptLinkedRefMap getConcepts() const { return m_concepts; }
+    virtual void setName(const QCString &name);
 
     virtual QCString title() const;
     virtual QCString compoundTypeString() const;
@@ -137,12 +137,13 @@ class NamespaceDefImpl : public DefinitionMixin<NamespaceDefMutable>
 
     void writeNamespaceDeclarations(OutputList &ol,const QCString &title,
             bool isConstantGroup=false);
+    void updateLocalName();
 
     QCString              fileName;
     FileList              files;
 
-    LinkedRefMap<const NamespaceDef> m_usingDirList;
-    LinkedRefMap<const ClassDef> m_usingDeclList;
+    LinkedRefMap<NamespaceDef> m_usingDirList;
+    LinkedRefMap<ClassDef> m_usingDeclList;
     LinkedRefMap<const Definition> m_innerCompounds;
 
     MemberLinkedRefMap    m_allMembers;
@@ -161,13 +162,13 @@ class NamespaceDefImpl : public DefinitionMixin<NamespaceDefMutable>
     bool                  m_inline = false;
 };
 
-NamespaceDefMutable *createNamespaceDef(const QCString &defFileName,int defLine,int defColumn,
+std::unique_ptr<NamespaceDef> createNamespaceDef(const QCString &defFileName,int defLine,int defColumn,
                  const QCString &name,const QCString &ref,
                  const QCString &refFile,const QCString &type,
                  bool isPublished)
 {
   //printf("createNamespaceDef(%s)\n",qPrint(name));
-  return new NamespaceDefImpl(defFileName,defLine,defColumn,name,ref,refFile,type,isPublished);
+  return std::make_unique<NamespaceDefImpl>(defFileName,defLine,defColumn,name,ref,refFile,type,isPublished);
 }
 
 //------------------------------------------------------------------
@@ -191,13 +192,13 @@ class NamespaceDefAliasImpl : public DefinitionAliasMixin<NamespaceDef>
     { return getNSAlias()->anchor(); }
     virtual int numDocMembers() const
     { return getNSAlias()->numDocMembers(); }
-    virtual const LinkedRefMap<const NamespaceDef> &getUsedNamespaces() const
+    virtual const LinkedRefMap<NamespaceDef> &getUsedNamespaces() const
     { return getNSAlias()->getUsedNamespaces(); }
-    virtual const LinkedRefMap<const ClassDef> &getUsedClasses() const
+    virtual const LinkedRefMap<ClassDef> &getUsedClasses() const
     { return getNSAlias()->getUsedClasses(); }
     virtual QCString displayName(bool b=TRUE) const
     { return makeDisplayName(this,b); }
-    virtual QCString localName() const
+    virtual const QCString &localName() const
     { return getNSAlias()->localName(); }
     virtual bool isConstantGroup() const
     { return getNSAlias()->isConstantGroup(); }
@@ -245,9 +246,9 @@ class NamespaceDefAliasImpl : public DefinitionAliasMixin<NamespaceDef>
     { return getNSAlias()->countVisibleMembers(); }
 };
 
-NamespaceDef *createNamespaceDefAlias(const Definition *newScope,const NamespaceDef *nd)
+std::unique_ptr<NamespaceDef> createNamespaceDefAlias(const Definition *newScope,const NamespaceDef *nd)
 {
-  NamespaceDef *alnd = new NamespaceDefAliasImpl(newScope,nd);
+  auto alnd = std::make_unique<NamespaceDefAliasImpl>(newScope,nd);
   //printf("alnd name=%s localName=%s qualifiedName=%s displayName()=%s\n",
   //    qPrint(alnd->name()),qPrint(alnd->localName()),qPrint(alnd->qualifiedName()),
   //    qPrint(alnd->displayName()));
@@ -297,6 +298,25 @@ NamespaceDefImpl::NamespaceDefImpl(const QCString &df,int dl,int dc,
   {
     m_type = NAMESPACE;
   }
+
+  updateLocalName();
+}
+
+void NamespaceDefImpl::updateLocalName()
+{
+  QCString locName=name();
+  int i=locName.findRev("::");
+  if (i!=-1)
+  {
+    locName=locName.mid(i+2);
+  }
+  setLocalName(locName);
+}
+
+void NamespaceDefImpl::setName(const QCString &name)
+{
+  DefinitionMixin<NamespaceDefMutable>::setName(name);
+  updateLocalName();
 }
 
 NamespaceDefImpl::~NamespaceDefImpl()
@@ -350,7 +370,7 @@ void NamespaceDefImpl::insertUsedFile(FileDef *fd)
   }
 }
 
-void NamespaceDefImpl::addInnerCompound(const Definition *d)
+void NamespaceDefImpl::addInnerCompound(Definition *d)
 {
   //printf("%s:NamespaceDefImpl::addInnerCompound(%s)\n",qPrint(name()),qPrint(d->name()));
   m_innerCompounds.add(d->localName(),d);
@@ -368,7 +388,7 @@ void NamespaceDefImpl::addInnerCompound(const Definition *d)
   }
 }
 
-void NamespaceDefImpl::insertClass(const ClassDef *cd)
+void NamespaceDefImpl::insertClass(ClassDef *cd)
 {
   ClassLinkedRefMap &d = classes;
 
@@ -391,12 +411,12 @@ void NamespaceDefImpl::insertClass(const ClassDef *cd)
   d.add(cd->name(),cd);
 }
 
-void NamespaceDefImpl::insertConcept(const ConceptDef *cd)
+void NamespaceDefImpl::insertConcept(ConceptDef *cd)
 {
   m_concepts.add(cd->name(),cd);
 }
 
-void NamespaceDefImpl::insertNamespace(const NamespaceDef *nd)
+void NamespaceDefImpl::insertNamespace(NamespaceDef *nd)
 {
   namespaces.add(nd->name(),nd);
 }
@@ -521,7 +541,7 @@ void NamespaceDefImpl::insertMember(MemberDef *md)
         std::unique_ptr<MemberDef> aliasMd;
         if (outerScope->definitionType()==Definition::TypeNamespace)
         {
-          aliasMd.reset(createMemberDefAlias(outerScope,md));
+          aliasMd = createMemberDefAlias(outerScope,md);
           NamespaceDefMutable *ndm = toNamespaceDefMutable(outerScope);
           if (ndm)
           {
@@ -530,7 +550,7 @@ void NamespaceDefImpl::insertMember(MemberDef *md)
         }
         else if (outerScope->definitionType()==Definition::TypeFile)
         {
-          aliasMd.reset(createMemberDefAlias(outerScope,md));
+          aliasMd = createMemberDefAlias(outerScope,md);
           toFileDef(outerScope)->insertMember(aliasMd.get());
         }
         if (aliasMd)
@@ -560,9 +580,11 @@ bool NamespaceDefImpl::hasDetailedDescription() const
 
 void NamespaceDefImpl::writeTagFile(TextStream &tagFile)
 {
+  QCString fn = getOutputFileBase();
+  addHtmlExtensionIfMissing(fn);
   tagFile << "  <compound kind=\"namespace\">\n";
   tagFile << "    <name>" << convertToXML(name()) << "</name>\n";
-  tagFile << "    <filename>" << addHtmlExtensionIfMissing(getOutputFileBase()) << "</filename>\n";
+  tagFile << "    <filename>" << fn << "</filename>\n";
   QCString idStr = id();
   if (!idStr.isEmpty())
   {
@@ -665,7 +687,6 @@ void NamespaceDefImpl::writeDetailedDescription(OutputList &ol,const QCString &t
       ol.pushGeneratorState();
         ol.disable(OutputType::Man);
         ol.disable(OutputType::RTF);
-        //ol.newParagraph(); // FIXME:PARA
         ol.enableAll();
         ol.disableAllBut(OutputType::Man);
         ol.enable(OutputType::Latex);
@@ -713,12 +734,6 @@ void NamespaceDefImpl::writeBriefDescription(OutputList &ol)
       ol.popGeneratorState();
       ol.endParagraph();
     }
-
-    // FIXME:PARA
-    //ol.pushGeneratorState();
-    //ol.disable(OutputType::RTF);
-    //ol.newParagraph();
-    //ol.popGeneratorState();
   }
 
   // Write a summary of the Slice definition including metadata.
@@ -1122,6 +1137,8 @@ void NamespaceDefImpl::writeQuickMemberLinks(OutputList &ol,const MemberDef *cur
       {
         if (md->isLinkableInProject())
         {
+          QCString fn = md->getOutputFileBase();
+          addHtmlExtensionIfMissing(fn);
           if (md==currentMd) // selected item => highlight
           {
             ol.writeString("          <tr><td class=\"navtabHL\">");
@@ -1133,7 +1150,7 @@ void NamespaceDefImpl::writeQuickMemberLinks(OutputList &ol,const MemberDef *cur
           ol.writeString("<a class=\"navtab\" ");
           ol.writeString("href=\"");
           if (createSubDirs) ol.writeString("../../");
-          ol.writeString(addHtmlExtensionIfMissing(md->getOutputFileBase())+"#"+md->anchor());
+          ol.writeString(fn+"#"+md->anchor());
           ol.writeString("\">");
           ol.writeString(convertToHtml(md->localName()));
           ol.writeString("</a>");
@@ -1167,13 +1184,13 @@ int NamespaceDefImpl::numDocMembers() const
   return (allMemberList ? allMemberList->numDocMembers() : 0) + static_cast<int>(m_innerCompounds.size());
 }
 
-void NamespaceDefImpl::addUsingDirective(const NamespaceDef *nd)
+void NamespaceDefImpl::addUsingDirective(NamespaceDef *nd)
 {
   m_usingDirList.add(nd->qualifiedName(),nd);
   //printf("%s: NamespaceDefImpl::addUsingDirective: %s:%zu\n",qPrint(name()),qPrint(nd->qualifiedName()),m_usingDirList.size());
 }
 
-void NamespaceDefImpl::addUsingDeclaration(const ClassDef *cd)
+void NamespaceDefImpl::addUsingDeclaration(ClassDef *cd)
 {
   m_usingDeclList.add(cd->qualifiedName(),cd);
 }
@@ -1234,23 +1251,12 @@ QCString NamespaceDefImpl::displayName(bool includeScope) const
   return makeDisplayName(this,includeScope);
 }
 
-QCString NamespaceDefImpl::localName() const
-{
-  QCString result=name();
-  int i=result.findRev("::");
-  if (i!=-1)
-  {
-    result=result.mid(i+2);
-  }
-  return result;
-}
-
 void NamespaceDefImpl::combineUsingRelations(NamespaceDefSet &visitedNamespaces)
 {
   if (visitedNamespaces.find(this)!=visitedNamespaces.end()) return; // already processed
   visitedNamespaces.insert(this);
 
-  LinkedRefMap<const NamespaceDef> usingDirList = m_usingDirList;
+  LinkedRefMap<NamespaceDef> usingDirList = m_usingDirList;
   for (auto &nd : usingDirList)
   {
     NamespaceDefMutable *ndm = toNamespaceDefMutable(nd);
@@ -1362,7 +1368,7 @@ void NamespaceLinkedRefMap::writeDeclaration(OutputList &ol,const QCString &titl
       if (lang==SrcLangExt_IDL && (isConstantGroup != nd->isConstantGroup()))
           continue; // will be output in another pass, see layout_default.xml
       ol.startMemberDeclaration();
-      ol.startMemberItem(nd->getOutputFileBase(),0);
+      ol.startMemberItem(nd->getOutputFileBase(),OutputGenerator::MemberItemType::Normal);
       QCString ct = nd->compoundTypeString();
       ol.docify(ct);
       ol.docify(" ");
@@ -1377,7 +1383,7 @@ void NamespaceLinkedRefMap::writeDeclaration(OutputList &ol,const QCString &titl
         name = nd->displayName();
       }
       ol.writeObjectLink(nd->getReference(),nd->getOutputFileBase(),QCString(),name);
-      ol.endMemberItem();
+      ol.endMemberItem(OutputGenerator::MemberItemType::Normal);
       if (!nd->briefDescription().isEmpty() && Config_getBool(BRIEF_MEMBER_DESC))
       {
         ol.startMemberDescription(nd->getOutputFileBase());
@@ -1613,18 +1619,6 @@ NamespaceDefMutable *toNamespaceDefMutable(Definition *d)
   }
 }
 
-NamespaceDefMutable *toNamespaceDefMutable(const Definition *d)
-{
-  if (d && typeid(*d)==typeid(NamespaceDefImpl))
-  {
-    return const_cast<NamespaceDefMutable*>(static_cast<const NamespaceDefMutable*>(d));
-  }
-  else
-  {
-    return 0;
-  }
-}
-
 // --- Helpers
 
 
