@@ -16,6 +16,7 @@
 #ifndef NAMESPACEDEF_H
 #define NAMESPACEDEF_H
 
+#include <memory>
 #include <set>
 
 #include "definition.h"
@@ -54,17 +55,15 @@ class NamespaceLinkedRefMap : public LinkedRefMap<const NamespaceDef>
 class NamespaceDef : public Definition
 {
   public:
-    virtual ~NamespaceDef() {}
     virtual DefType definitionType() const = 0;
 
     // ---- getters
     virtual QCString getOutputFileBase() const = 0;
     virtual QCString anchor() const = 0;
     virtual int numDocMembers() const = 0;
-    virtual LinkedRefMap<const NamespaceDef> getUsedNamespaces() const = 0;
-    virtual LinkedRefMap<const ClassDef> getUsedClasses() const = 0;
+    virtual const LinkedRefMap<NamespaceDef> &getUsedNamespaces() const = 0;
+    virtual const LinkedRefMap<ClassDef> &getUsedClasses() const = 0;
     virtual QCString displayName(bool=TRUE) const = 0;
-    virtual QCString localName() const = 0;
     virtual bool isConstantGroup() const = 0;
     virtual bool isModule()        const = 0;
     virtual bool isLibrary() const = 0;
@@ -77,6 +76,7 @@ class NamespaceDef : public Definition
     virtual MemberList *getMemberList(MemberListType lt) const = 0;
     virtual const MemberLists &getMemberLists() const = 0;
     virtual const MemberDef *getMemberByName(const QCString &) const = 0;
+    virtual int countVisibleMembers() const = 0;
 
     /*! Returns the user defined member groups */
     virtual const MemberGroupList &getMemberGroups() const = 0;
@@ -114,33 +114,34 @@ class NamespaceDefMutable : public DefinitionMutable, public NamespaceDef
     virtual void writeMemberPages(OutputList &ol) = 0;
     virtual void writeQuickMemberLinks(OutputList &ol,const MemberDef *currentMd) const = 0;
     virtual void writeTagFile(TextStream &) = 0;
-    virtual void insertClass(const ClassDef *cd) = 0;
-    virtual void insertConcept(const ConceptDef *cd) = 0;
-    virtual void insertNamespace(const NamespaceDef *nd) = 0;
-    virtual void insertMember(MemberDef *md) = 0; // md cannot be const, since setSectionList is called on it
+    virtual void insertClass(ClassDef *cd) = 0;
+    virtual void insertConcept(ConceptDef *cd) = 0;
+    virtual void insertNamespace(NamespaceDef *nd) = 0;
+    virtual void insertMember(MemberDef *md) = 0;
     virtual void computeAnchors() = 0;
     virtual void countMembers() = 0;
     virtual void addMembersToMemberGroup() = 0;
     virtual void distributeMemberGroupDocumentation() = 0;
     virtual void findSectionsInDocumentation() = 0;
     virtual void sortMemberLists() = 0;
-    virtual void addInnerCompound(const Definition *d) = 0;
+    virtual void addInnerCompound(Definition *d) = 0;
     virtual void addListReferences() = 0;
     virtual void setFileName(const QCString &fn) = 0;
     virtual void combineUsingRelations(NamespaceDefSet &visitedNamespace) = 0;
-    virtual void addUsingDirective(const NamespaceDef *nd) = 0;
-    virtual void addUsingDeclaration(const ClassDef *cd) = 0;
+    virtual void addUsingDirective(NamespaceDef *nd) = 0;
+    virtual void addUsingDeclaration(ClassDef *cd) = 0;
     virtual void setInline(bool isInline) = 0;
+    virtual void setName(const QCString &name) = 0;
 };
 
 /** Factory method to create new NamespaceDef instance */
-NamespaceDefMutable *createNamespaceDef(const QCString &defFileName,int defLine,int defColumn,
+std::unique_ptr<NamespaceDef> createNamespaceDef(const QCString &defFileName,int defLine,int defColumn,
                  const QCString &name,const QCString &ref=QCString(),
                  const QCString &refFile=QCString(),const QCString &type=QCString(),
                  bool isPublished=false);
 
 /** Factory method to create an alias of an existing namespace. Used for inline namespaces. */
-NamespaceDef *createNamespaceDefAlias(const Definition *newScope, const NamespaceDef *nd);
+std::unique_ptr<NamespaceDef> createNamespaceDefAlias(const Definition *newScope,const NamespaceDef *nd);
 
 
 // --- Cast functions
@@ -149,7 +150,6 @@ NamespaceDef            *toNamespaceDef(Definition *d);
 NamespaceDef            *toNamespaceDef(DefinitionMutable *d);
 const NamespaceDef      *toNamespaceDef(const Definition *d);
 NamespaceDefMutable     *toNamespaceDefMutable(Definition *d);
-NamespaceDefMutable     *toNamespaceDefMutable(const Definition *d);
 
 // --- Helpers
 
